@@ -49,7 +49,6 @@
 
 #include "dnd.h"
 
-#include "bookmarks.h"
 #include "cpy.h"
 #include "rcp.h"
 #include "icons.h"
@@ -620,7 +619,7 @@ void on_drag_data (GtkWidget * widget, GdkDragContext * context, gint x, gint y,
 	for(tmp = list; tmp != NULL; tmp = tmp->next)
 	{
 	    u = tmp->data;
-	    add2bookmarks(treeview, u->url);
+	    BOOK_add2bookmarks(treeview, u->url);
 	    TRACE("DBG: adding %s to bookmarks\n",u->url);
 	}
 	list = uri_free_list(list);
@@ -914,8 +913,8 @@ void on_drag_data_get (GtkWidget * widget, GdkDragContext * context, GtkSelectio
 		selection_list_t *sl;
 		sl = (selection_list_t *) tmp->data;
 		if (!gtk_tree_row_reference_valid(sl->reference)) return;
-		sprintf(files, "file:/%s\r\n", sl->en->path);
-		files += (strlen(sl->en->path) + strlen("file:/") + 2);
+		sprintf(files, "file:%s\r\n", sl->en->path);
+		files += (strlen(sl->en->path) + strlen("file:") + 2);
 	    }
 	    
 	    break;
