@@ -49,14 +49,12 @@
 #include "goto.h"
 
 #include "dnd.h"
-#include "recent.h"
 #include "entry.h"
 #include "icons.h"
 #include "input.h"
 #include "menu.h"
 #include "monitor.h"
 #include "misc.h"
-#include "trash.h"
 #include "treestore.h"
 #include "widgets.h"
 
@@ -249,9 +247,9 @@ int go_to (treestuff_t *treestuff, const gchar *path)
     /*unselect_double_treeview(treeview);*/
     save_to_go_history((char *)path);
 #ifdef DEBUG
-    printf("DBG: add2history(%s)\n",(char *)path);
+    printf("DBG: RECENT_add2history(%s)\n",(char *)path);
 #endif
-   if (path) add2history((char *)path);
+   if (path) RECENT_add2history((char *)path);
 
     redlight=FALSE;
     return TRUE;
