@@ -106,6 +106,7 @@ static int childFD;
 static xfdir_t fstat_xfdir;
 static pid_t Gpid;
 static GtkTreeView *fstab_treeview;
+static  GtkTreeRowReference *fstab_reference=NULL;
 
 /* exported prototypes */
 int is_mounted(const gchar *mnt_point);
@@ -177,21 +178,52 @@ static void fork_finished_function(pid_t pid)
     GtkTreeIter iter;
     sprintf(m, "%d\n", pid);
 
-    tree_details->tubo_object = NULL;
     hide_stop();
     cursor_reset();
     /* another wait just in case the one in tubo misses 
      * (shouldn't happen, but it does)*/
-    
+    TRACE("fork_finished_function");
     waitpid(pid, &status, WNOHANG);
     en= get_selected_entry(&iter);
+    if (!en) {
+	TRACE("fork_finished_function all done");
+	if (fstab_reference) {
+	    gtk_tree_row_reference_free(fstab_reference);
+	    fstab_reference=NULL;
+	}
+	tree_details->tubo_object = NULL;
+	return;
+    }
     status=is_mounted((const gchar *)en->path);
     if (status < 0) TOGGLE_MOUNTED(en->type);
     else {
 	    if (status) SET_MOUNTED(en->type);
 	    else UNSET_MOUNTED(en->type);
     }
-    if (IS_MOUNTED(en->type))
+    if (fstab_treeview && fstab_reference && gtk_tree_row_reference_valid(fstab_reference)) {
+    	GtkTreeModel *treemodel = gtk_tree_view_get_model(fstab_treeview);
+      	GtkTreePath *treepath = gtk_tree_row_reference_get_path(fstab_reference);
+	if (treepath && gtk_tree_model_get_iter (treemodel,&iter,treepath)){
+	    if (IS_MOUNTED(en->type)){
+		int i;
+		for (i=0; i<TREECOUNT; i++){
+		    gtk_tree_selection_unselect_all(tree_details->treestuff[i].selection);
+		}
+		gtk_tree_selection_select_iter(gtk_tree_view_get_selection(fstab_treeview),&iter);
+		on_refresh((GtkButton *)(tree_details->window),NULL);
+	    }
+	    else  {
+		gtk_tree_view_collapse_row(fstab_treeview, treepath);
+		prune_row(treemodel, &iter, NULL,en);
+		insert_dummy_row(treemodel, &iter, NULL,en,NULL,NULL);
+		update_icon(treemodel,&iter);
+	    }
+	    gtk_tree_path_free(treepath);
+	}
+	gtk_tree_row_reference_free(fstab_reference);
+	fstab_reference=NULL;
+    }
+    /*if (IS_MOUNTED(en->type))
 	on_refresh((GtkButton *)(tree_details->window),NULL);
     else {
     	GtkTreePath *treepath;
@@ -202,11 +234,13 @@ static void fork_finished_function(pid_t pid)
 	insert_dummy_row(treemodel, &iter, NULL,en,NULL,NULL);
     	gtk_tree_path_free(treepath);
 	update_icon(treemodel,&iter);
-    }
+    }*/
     
 #ifdef DEBUG  
     printf("DBG:fstab over by fork_finished_function\n");
 #endif
+    tree_details->tubo_object = NULL;
+
 }
 
 /* NOTE: linux will fail if HAVE_STATFS is before HAVE_GETMNTENT section */
@@ -289,7 +323,7 @@ G_MODULE_EXPORT
 int
 fstab_mount (GtkTreeView *treeview, const gchar *mnt_point, gboolean umnt,tree_entry_t *en)
 {
-    gchar *argument[12];
+    gchar *argument[15];
     gchar **ap;
     int i;
     gchar *g=NULL;
@@ -301,21 +335,37 @@ fstab_mount (GtkTreeView *treeview, const gchar *mnt_point, gboolean umnt,tree_e
     static gchar *smb_browse=NULL;
     /*GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);*/
 
+    if (tree_details->tubo_object){
+	print_status("xfce/error",_(strerror(EBUSY)),NULL);
+	return 0;
+    }
+    if (tree_details->window){
+	GtkTreePath *tpath;
+	GtkTreeIter iter;
+	get_selected_entry(&iter);
+	tpath = gtk_tree_model_get_path(tree_details->treestuff[get_active_tree_id()].treemodel, &iter);
+	fstab_reference = gtk_tree_row_reference_new(tree_details->treestuff[get_active_tree_id()].treemodel, tpath);
+	gtk_tree_path_free(tpath);
+    }
+
     g_free(smb_browse);
     smb_browse=NULL;
     /* this will set the is_smb variable: */
-    if (!is_in_fstab(mnt_point)) {
-	if (!is_smb) {
-	    print_diagnostics("xfce/error",strerror(ENODEV),":\n","no smbfs support detected in kernel\n",NULL);
-	    return 0;
-	}
+    
+    if (!is_in_fstab(mnt_point) && !is_mounted(mnt_point)) {
 	if (en && IS_XF_NETSHARE(en->subtype))
 	{	    
+	    is_smb=TRUE;
 	    smb_browse=randomTmpName(NULL);
 	    unlink(smb_browse);
 	    print_diagnostics(NULL,"$mkdir ",smb_browse,"\n",NULL);
 	    mkdir(smb_browse, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
 	}
+	else {
+	    print_diagnostics("xfce/error",strerror(ENODEV),":\n","unknown fs type\n",NULL);
+	    return 0;
+    
+	}
     }
     
     if (getenv("XFFM_USE_SUDO") && strlen(getenv("XFFM_USE_SUDO"))) use_sudo=TRUE;
@@ -335,8 +385,7 @@ fstab_mount (GtkTreeView *treeview, const gchar *mnt_point, gboolean umnt,tree_e
     }
 		 
     
-    if (smb_browse) i=0;
-    else i=is_mounted(mnt_point);
+    if (smb_browse) i=0; else i=is_mounted(mnt_point);
     
     if (i >= 0){
       if (i) *ap++ = umount;
@@ -347,8 +396,12 @@ fstab_mount (GtkTreeView *treeview, const gchar *mnt_point, gboolean umnt,tree_e
     }
     if (is_smb && i==0) {
 	if (smb_browse) {
+	    static char options[64];
+	    sprintf(options,"uid=%d,gid=%d",(int)getuid(),(int)getgid());
 	    *ap++="-t";
 	    *ap++="smbfs";
+	    *ap++="-o";
+	    *ap++=options; 
 	}
 	smb_user=get_smbuserpass(tree_details->window,en);
 	if (!smb_user || !strlen(smb_user)){
@@ -404,6 +457,10 @@ fstab_mount (GtkTreeView *treeview, const gchar *mnt_point, gboolean umnt,tree_e
     	
     Gpid = TuboPID(tree_details->tubo_object);
     g_free(smb_options);
+    while (tree_details->tubo_object){
+	while (gtk_events_pending()) gtk_main_iteration();
+	usleep(100);
+    }
     if (smb_browse) {
 	go_to(tree_details->treestuff+get_tree_id(treeview),smb_browse);
     }
@@ -645,6 +702,7 @@ is_in_fstab (const gchar *path)
 #define MNTTYPE_HSFS	"hsfs"
 #define MNTTYPE_NFS	"nfs"
 #define MNTTYPE_SWAP	"swap"
+#define MNTTYPE_SMB	"smbfs"
 
 int
 open_fstab (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath, 
@@ -669,6 +727,9 @@ open_fstab (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *treepath,
 		if(strcmp(MNTTYPE_SWAP, vbuf.vfs_fstype) == 0)
 			continue;
 
+		if(strcmp(MNTTYPE_SMB, vbuf.vfs_fstype) == 0)
+			is_smb=TRUE;
+		
 		if (!g_file_test(vbuf.vfs_mountp, G_FILE_TEST_IS_DIR))
 			continue;
 
