@@ -17,6 +17,7 @@
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
+#ifdef USE_SMB_BRANCH
 
 #include <unistd.h>
 #include <stdarg.h>
@@ -36,12 +37,16 @@
 
 #include "constants.h"
 #include "types.h"
+#include "smb_module.h"
+
+#include "basic_row.h"
+
+#include "smb_download.h"
 
-#include "add_folder.h"
-#include "add_node_contents.h"
 #include "cpy.h"
 #include "uri.h"
 #include "entry.h"
+#include "remove.h"
 #include "monitor.h"
 #include "misc.h"
 #include "smb_list.h"
@@ -49,6 +54,7 @@
 #include "smb_open.h"
 #include "smblookup.h"
 #include "tubo.h"
+#include "password_dialog.h"
 
 
 
@@ -58,7 +64,7 @@ extern void *smb_object;
 
 
 static int SMBResult;
-static GtkWidget *countW=NULL,*count_label;
+static GtkWidget *countW=NULL,*count_label, *count_progress;
 
 
 static void
@@ -83,10 +89,8 @@ SMBFork (void *data)
    the_user=strtok(NULL,"\n");
    if (!the_user) _exit(123); /* error processing */
    the_command=the_user+strlen(the_user)+1;
-#ifdef DEBUG  
-   fprintf(stderr,"smbclient %s -U %s\n",the_netbios,the_user);
-   fprintf(stderr,"%s\n",the_command); fflush(NULL);sleep(1);
-#endif
+   DBG("smbclient %s -U %s\n",the_netbios,the_user);
+   DBG("%s\n",the_command); fflush(NULL);sleep(1);
      /*read data from tmpfile */  
    execlp ("smbclient", "smbclient", the_netbios,"-U",the_user, "-c", the_command, (char *) 0);
   _exit(123); /* error processing */
@@ -101,12 +105,14 @@ SMBStdout (int n, void *data)
   int i;
   if (n) return TRUE;		/* this would mean binary data */
   line = (char *) data;
-  print_diagnostics (smb_treeview,NULL,line,NULL);
+  /* this should be nonverbose until there is a progress dialog for
+   * uploading and deleted similar to the download dialog.
+   * This new dialog may be entirely unnecessary... */
+  print_diagnostics ("nonverbose",line,NULL);
   
   for (i=0;smb_errors[i]!=NULL;i++){
       if (strstr (line, smb_errors[i]))  {
-	  print_diagnostics(smb_treeview,"xf_ERROR_ICON",
-			strerror(EPERM),NULL);
+	  print_diagnostics("xfce/error", strerror(EPERM),NULL);
       }	
   }
   return TRUE;
@@ -126,24 +132,82 @@ SMBForkOver (pid_t pid)
   {
   case CHALLENGED:
     /*printf("dbg:%s\n",_("File download failed."));*/
-    print_status(smb_treeview,"xf_ERROR_ICON",_("File download failed."),NULL);
+    print_status("xfce/error",_("File download failed."),NULL);
     break;
   default:
     /* upload was successful: */
     /*printf("dbg:%s\n",_("Download done."));*/
-    /*xf_dlg_error(SMBparent,"smbclient",_("Download done."));*/
+    /*xfce/dlg_error(SMBparent,"smbclient",_("Download done."));*/
     break;
 
   }
   smb_object = NULL;
-  cursor_reset (smb_treeview);
-  local_monitor(smb_treeview,TRUE);//FIXME
+  cursor_reset ();
+  local_monitor(TRUE);
 }
 
 
+
+#define TIMERVAL 250
+
+static int download_timer;
+static int download_count;
+static GList *download_list=NULL;
+
 static void cb_count_destroy(GtkWidget *widget,gpointer data){
-	countW=NULL;
+    GList *tmp;
+    for (tmp=download_list; tmp; tmp=tmp->next){
+	g_free(tmp->data);
+    }
+    g_list_free(download_list);
+    download_list=NULL;
+    countW=NULL;
 }
+
+static
+gint download_watch(GtkTreeView * treeview){
+    gchar *s="-\\|/";
+    static int s_pos=0;
+    struct stat st;
+    long size=0;
+    int n=download_count-g_list_length(download_list)+1;
+    static gchar *str=NULL;
+    static gchar *str_n=NULL;
+    gfloat fraction;
+    TRACE("at timer");
+    if (!download_list) return FALSE;
+
+    if (!g_file_test((gchar *) download_list->data,G_FILE_TEST_IS_DIR) 
+	    && stat( (gchar *) download_list->data, &st)==0){
+	size=st.st_size;
+    }
+    g_free(str);
+    g_free(str_n);
+
+    if (download_count==0) fraction=1.0; else fraction=(float)n/(float)download_count;
+    str_n=g_strdup_printf("%d/%d",n,download_count);
+    if (size) str=g_strdup_printf("%s (%ld)",(gchar *) download_list->data,(long)size);
+    else {
+	str=g_strdup_printf("%s (%c)",(gchar *) download_list->data,s[s_pos++]);
+	if (s_pos >= strlen(s)) s_pos=0;
+    }
+	
+    gtk_label_set_text ((GtkLabel *)count_label,(const gchar *)str);
+    gtk_progress_bar_set_fraction((GtkProgressBar *) count_progress, fraction);
+    gtk_progress_bar_set_text((GtkProgressBar *) count_progress, (const gchar *)str_n);
+
+    TRACE("%s (fraction=%f): %s",str_n,fraction,str);
+    /* if next file has appeared, this one is done. 
+     * )this will give false readings if the next file happens to be a directory
+     * which will be overwritten, this is a not fixable buglet).*/
+    if (download_list->next && g_file_test( (gchar *) download_list->next->data, G_FILE_TEST_EXISTS)){
+	gconstpointer g = download_list->data;
+	download_list=g_list_remove(download_list,g);
+    }	
+    return TRUE;
+}
+
+
 static void download_window(GtkWidget *parent,char *host){
  	
   countW=gtk_dialog_new ();
@@ -155,44 +219,52 @@ static void download_window(GtkWidget *parent,char *host){
   count_label = gtk_label_new (host);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (countW)->vbox), count_label, TRUE, TRUE, 3);
   count_label = gtk_label_new (".............................................................");
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (countW)->action_area), count_label, TRUE, TRUE, 3);
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (countW)->vbox), count_label, TRUE, TRUE, 3);
+  count_progress=gtk_progress_bar_new ();
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (countW)->action_area), count_progress, TRUE, TRUE, 3);
+  
+
   gtk_widget_realize (countW);
   if (parent) gtk_window_set_transient_for (GTK_WINDOW (countW), GTK_WINDOW (parent)); 
   g_signal_connect (G_OBJECT (countW), "destroy", GTK_SIGNAL_FUNC (cb_count_destroy), NULL);
   gtk_widget_show_all(countW);
   gdk_flush();
+  download_timer = g_timeout_add_full(0, TIMERVAL, 
+			(GtkFunction) download_watch, NULL, NULL);
   return ;
 }
 
 
+G_MODULE_EXPORT
 void
 SMBGetFile (GtkTreeView *treeview, char *target,GList *list)
 {
   char *dndS,*host=NULL,*user,*orig_share=NULL,*share,*file,*filename=NULL;
   uri *u;
   gchar *s;
-  static char *fname;
+  gchar *fname;
   FILE *tmpfile=NULL;
   char *w;
   int i;
-  gchar *SMBtmpfile;
+  static gchar *SMBtmpfile=NULL;
   gboolean first=TRUE,isdir;
-  tree_details_t *tree_details = get_tree_details(treeview);
   gboolean samba_server=FALSE;
     
 
   smb_treeview=treeview;
   s=g_find_program_in_path("smbclient");
   if (!s){
-	  print_status(treeview,"xf_ERROR_ICON",strerror(ENOENT),":","smbclient",NULL);
+	  print_status("xfce/error",strerror(ENOENT),":","smbclient",NULL);
 	  return;
   }
 
   if ((fname=randomTmpName(NULL))==NULL) return;
     if ((tmpfile=fopen(fname,"w"))==NULL) {
-	print_status(treeview,"xf_ERROR_ICON",strerror(EPERM),":",(fname)?fname:"/tmp/?",NULL);
+	print_status("xfce/error",strerror(EPERM),":",(fname)?fname:"?",NULL);
+	g_free(fname);
 	return;
   }
+ download_count=0;
   
  for (;list!=NULL;list=list->next){
   u = list->data;
@@ -216,8 +288,7 @@ SMBGetFile (GtkTreeView *treeview, char *target,GList *list)
   if (strncmp("smb://",dndS,strlen("smb://")!=0)&&
       strncmp("SMB://",dndS,strlen("SMB://")!=0)		  ) {
 	incorrect_DND:
-	print_status(treeview,"xf_ERROR_ICON",_("Incorrect DnD specification"),
-			NULL);
+	print_status("xfce/error",strerror(EINVAL), NULL);
 	return;
   }
   if (strncmp("SMB://",dndS,strlen("SMB://"))==0) samba_server=TRUE;
@@ -259,48 +330,63 @@ SMBGetFile (GtkTreeView *treeview, char *target,GList *list)
   }
   /* only process files from first drop */
   if (orig_share && strcmp(share,orig_share)!=0) {
-	  print_diagnostics(treeview,"xf_ERROR_ICON",
-			  _("Only drops from a single share are allowed"),
-			  NULL);
+	  print_diagnostics("xfce/error", _("Only drops from a single share are allowed"), NULL);
 	  g_free(orig_share);orig_share=NULL;
 	  return;
   }
   {
     gchar *local_file=g_strdup(filename);
+    gchar *tgt=g_build_filename(target,local_file,NULL);
     if (!samba_server) ascii_readable(local_file);
-   print_diagnostics(treeview,"xf_INFO_ICON","get ",
-		      file,local_file,";\n",NULL);
+   print_diagnostics(NULL,"get ", file," ",local_file,";\n",NULL);
    if (isdir){
       fprintf(tmpfile,
 	"lcd \"%s\";cd \"/%s\";cd ..;prompt;recurse; mget \"%s\";recurse;prompt;cd /;",
 	target,file,local_file);
     } else {
+      struct stat st;
+      stat (tgt,&st);
       fprintf(tmpfile,"lcd \"%s\";get \"%s\" \"%s\";",
 			    target,file,local_file); 
-			    /*target,file,filename); */
+      /* trash whatever exists so that download monitoring can work. */
+      /* I exclude fifo's for double click mechanics. */
+#ifdef USE_NPIPE
+      if (!S_ISFIFO(st.st_mode))
+#endif
+      {
+	if (getenv("XFFM_DEFAULT_UNLINK") && strcmp(getenv("XFFM_DEFAULT_UNLINK"),"unlink")==0){
+	    DBG("unlinking %s",tgt);
+	    unlinkit(tgt);
+	} else {
+	    DBG("trashing %s",tgt);
+	    wasteit(tgt);
+	}
+      }
     }
+    download_list = g_list_append(download_list, tgt);
+    download_count++;
     g_free(local_file);local_file=NULL;
   }
  } /* end for list elements */
  
   fclose(tmpfile);
-  SMBtmpfile=g_strdup(fname); 
+  if (SMBtmpfile) g_free(SMBtmpfile);
+  SMBtmpfile=fname; 
   if (filename) {
 	  g_free(filename);
-	  filename=NULL;
   }
   filename=NULL;
   /* wait until OK to proceed */
-  cursor_wait (treeview);
+  cursor_wait ();
   smb_wait(FALSE);
   download_window(tree_details->window,host);
   SMBResult=0;
   smb_object = Tubo (SMBFork,
 		  (void *)SMBtmpfile,
 		  SMBForkOver, 
-		  TRUE, 
+		  NULL, 
 		  SMBStdout, 
-		  smb_stderr);
+		  smb_stderr,0,FALSE);
   if (orig_share) {
 	  g_free(orig_share);
 	  orig_share=NULL;
@@ -316,17 +402,15 @@ SMBDropForkOver (pid_t pid)
 {
   while (gtk_events_pending()) gtk_main_iteration();
   gdk_flush();
-  cursor_reset (smb_treeview);
+  cursor_reset ();
   switch (SMBResult)
   {
   case CHALLENGED:
-    print_status (smb_treeview,"xf_ERROR_ICON",
-		    strerror(EPERM),
-		    NULL);
+    print_status ("xfce/error", strerror(EPERM), NULL);
     break;
   default:
     /* upload was successful: */
-    print_status (smb_treeview,"xf_INFO_ICON",_("Command done"),NULL);
+    print_status ("xfce/info",_("Command done"),NULL);
     break;
 
   }
@@ -371,13 +455,12 @@ SMBDropFork (void *data)
 	  strcat(argv[5],w);	  
   }
  
-#ifdef DEBUG  
-   fprintf(stderr,"%s %s %s %s %s %s\n",argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]); fflush(NULL);sleep(1);
-#endif
+   DBG("%s %s %s %s %s %s\n",argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]); fflush(NULL);sleep(1);
   execvp (argv[0],argv);
 }
 
 
+G_MODULE_EXPORT
 void
 SMBDropFile (GtkTreeView *treeview,tree_entry_t *t_en, 
 		GtkTreeIter *iter,char *tmpfile)
@@ -385,6 +468,7 @@ SMBDropFile (GtkTreeView *treeview,tree_entry_t *t_en,
   char *argv[12];
   gchar *g;
   
+    GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
   if (!IS_XF_NETSHARE(t_en->subtype) && !IS_NETFILE(t_en->subtype) && !IS_NETDIR(t_en->subtype))
 	  return;
   init_smb_list(treeview,t_en->path,SHOWS_HIDDEN(t_en->type));
@@ -407,31 +491,31 @@ SMBDropFile (GtkTreeView *treeview,tree_entry_t *t_en,
   argv[5]=tmpfile;
   argv[6]=0;
   
-  /*stopcleanup = FALSE;*/
-  print_diagnostics (treeview,"xf_INFO_ICON",_("Copying"),"...","\n",NULL);
-  print_status (treeview,"xf_INFO_ICON",_("Copying"),"...",NULL);
+  print_diagnostics ("nonverbose",_("Copying"),"...","\n",NULL);
+  print_status (NULL,_("Copying"),"...",NULL);
   
   
   /* wait until OK to proceed */
-  cursor_wait (treeview);
+  cursor_wait ();
   smb_wait(FALSE);
   
   smb_object = Tubo (SMBDropFork, 
 		  (void *)argv,
 		  SMBDropForkOver, 
-		  TRUE, 
+		  NULL, 
 		  SMBListStdout,
-		  smb_stderr);
+		  smb_stderr,0,FALSE);
 
   smb_wait(TRUE);
   g_free(g);g=NULL;
 
-  remove_folder(treeview,iter);
+  prune_row(treemodel, iter,NULL,t_en);
+  /*insert_dummy_row(treemodel, iter,NULL,en,NULL,NULL);*/
   add_smb_stuff(treeview,iter,t_en->tag);
   SET_LOADED(t_en->type);
   UNSET_EXPANDED(t_en->type);
   unlink(tmpfile);
-  cursor_reset (treeview);
+  cursor_reset ();
   return;
 }
 
@@ -447,12 +531,15 @@ SMBrmStdout (int n, void *data)
   if (n)
     return TRUE;		/* this would mean binary data */
   line = (char *) data;
-  print_diagnostics (smb_treeview,NULL,line,NULL);
+  /* this should be nonverbose until there is a progress dialog for
+   * uploading and deleted similar to the download dialog.
+   * This new dialog may be entirely unnecessary... */
+  print_diagnostics ("nonverbose",line,NULL);
   for (i=0;smb_errors[i]!=NULL;i++){
       if (strstr (line, smb_errors[i]))  {
 	    gchar *f,*g;
 	    GList *tmp;
-	    if (!strchr(line,'\\')) g_assert_not_reached();
+	    if (!strchr(line,'\\')) assert_not_reached();
 	    f=g_strdup(strchr(line,'\\')+1);
 	    for (g=f;*g;g++) if (*g=='\\') *g='/';
 	    for (tmp=local_remove_list;tmp;tmp = tmp->next)
@@ -464,8 +551,7 @@ SMBrmStdout (int n, void *data)
 	      if (strstr(en->path,g)){
 	      	 local_remove_list = g_list_remove(local_remove_list, tmp->data);
 	    	 gtk_tree_row_reference_free(reference);
-	   	 print_diagnostics(smb_treeview,"xf_ERROR_ICON",
-			strerror(EPERM),": rm ",en->path,NULL);
+	   	 print_diagnostics("xfce/error", strerror(EPERM),": rm ",en->path,NULL);
 		 break;
 	      }
 	    }
@@ -476,6 +562,7 @@ SMBrmStdout (int n, void *data)
 }
 
 
+G_MODULE_EXPORT
 GList *
 SMBrmFile (GtkTreeView *treeview,char *remote_location,
 	       char *remote_pass,char *tmpfile,GList *reference_list)
@@ -494,37 +581,39 @@ SMBrmFile (GtkTreeView *treeview,char *remote_location,
   argv[5]=tmpfile;
   argv[6]=0;
   
-  /*stopcleanup = FALSE;*/
-  print_diagnostics (treeview,"xf_WARNING_ICON",_("Removing"),"...","\n",NULL);
-  print_status (treeview,"xf_WARNING_ICON",_("Removing"),"...",NULL);
+  print_diagnostics ("nonverbose",_("Removing"),"...","\n",NULL);
+  print_status ("xfce/warning",_("Removing"),"...",NULL);
   
   
   /* wait until OK to proceed */
-  cursor_wait (treeview);
+  cursor_wait ();
   smb_wait(FALSE);
   
   smb_object = Tubo (SMBDropFork, 
 		  (void *)argv,
 		  SMBDropForkOver, 
-		  TRUE, 
+		  NULL, 
 		  SMBrmStdout,
-		  smb_stderr);
+		  smb_stderr,0,FALSE);
 
   smb_wait(TRUE);
-  cursor_reset (treeview);
+  cursor_reset ();
 
   return local_remove_list;
 }
 
 
+G_MODULE_EXPORT
 void
 SMBmkdir (GtkTreeView *treeview,tree_entry_t *t_en,
-	       GtkTreeIter *iter,char *name)
+	       GtkTreeIter *iter,gchar *name)
 {
   char *argv[12];
   gchar *g=NULL,*target=NULL;
   gchar *command=NULL;
 
+  GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
+
   if (!IS_XF_NETSHARE(t_en->subtype) && 
 	!IS_NETDIR(t_en->subtype))  return;
   init_smb_list(treeview,t_en->path,SHOWS_HIDDEN(t_en->type));
@@ -558,42 +647,103 @@ SMBmkdir (GtkTreeView *treeview,tree_entry_t *t_en,
   argv[6]=0;
   
    
-  /*stopcleanup = FALSE;*/
-  print_diagnostics (treeview,"xf_WARNING_ICON",
-		  "mkdir",t_en->path,"/",name,"\n",NULL);
-  print_status (treeview,"xf_WARNING_ICON","mkdir...",NULL);
+  print_diagnostics ("nonverbose", "mkdir",t_en->path,"/",name,"\n",NULL);
+  print_status ("xfce/warning","mkdir...",NULL);
 
 #ifdef DEBUG  
-   print_diagnostics (treeview,"xf_ERROR_ICON",
+   print_diagnostics ("nonverbose",
 		  argv[0]," ",argv[1]," ",argv[2]," ",argv[3]," ",
 		  argv[4]," ",argv[5],"\n",NULL); 
   
-#else
+#endif
   
   /* wait until OK to proceed */
-  cursor_wait (treeview);
+  cursor_wait ();
   smb_wait(FALSE);
+  reset_dummy_row(treemodel,iter,NULL,NULL,"xfce/warning",_("Loading..."));
   
   smb_object = Tubo (fork_function, 
 		  (void *)argv,
 		  SMBDropForkOver, 
-		  TRUE, 
+		  NULL, 
 		  SMBListStdout,
-		  smb_stderr);
+		  smb_stderr,0,FALSE);
 
   smb_wait(TRUE);
   g_free(g);g=NULL;
   g_free(command);command=NULL;
-  remove_folder(treeview,iter);
+  prune_row(treemodel,iter,NULL,t_en);
+  /* insert_dummy_row(treemodel,iter,NULL,t_en,NULL,NULL);*/
   add_smb_stuff(treeview,iter,t_en->tag);
   SET_LOADED(t_en->type);
   UNSET_EXPANDED(t_en->type);
-  cursor_reset (treeview);
-#endif
+  cursor_reset ();
   return;
 }
 
+G_MODULE_EXPORT
+const gchar *get_netfile_cache_dir(void){
+    gchar *xdg_dir;
+    static gchar *cache_dir=NULL;
+    if (cache_dir) return (const gchar *)cache_dir;
+
+    xdg_dir=xfce_resource_save_location (XFCE_RESOURCE_CACHE,"/",TRUE);
+    cache_dir = g_build_filename(xdg_dir,"xfce4","xffm","smb",NULL);  
+    g_free(xdg_dir);
+    mkdir(cache_dir, 0770);
+    if (!g_file_test(cache_dir,G_FILE_TEST_IS_DIR)) {
+	g_free(cache_dir);
+	cache_dir=NULL;
+	return NULL;
+    }
+    
+    return (const gchar *)cache_dir;
+}
 
 
+G_MODULE_EXPORT
+const gchar *SMBget_cache_file(GtkTreeView *treeview, tree_entry_t *en){
+	gchar *server,*remote_file;
+	static gchar *file=NULL;
+	GList *list=NULL;
+	DBG("%s is netfile",en->path);
+	
+	/* download */
+	server=g_strdup(en->path+2);
+	strtok(server,"/");
+	remote_file=server+strlen(server)+1;
+	TRACE("%s is netfile",remote_file);
+	g_free(file);
+	file = g_strdup_printf ("%s://%s@%s:%s\n",
+		  IS_SAMBA_SERVER(en->subtype)?"SMB":"smb",
+		  (en->tag)?en->tag:"GUEST%%",
+		  server,remote_file);
+	remote_file=g_path_get_basename(remote_file);
+	g_free(server);
+	uri_parse_list(file, &list);
+	g_free(file);
+	file=g_build_filename(get_netfile_cache_dir(),remote_file,NULL);
+	g_free(remote_file);
+#ifdef USE_NPIPE
+#undef USE_NPIPE
+#endif
+#ifdef USE_NPIPE
+	/* using named pipes does not work too well, because read/write
+	 * is blocking. Thus the application may block depending on
+	 * unpredictable user response...
+	 * thus it is avoided for 4.2.x*/
+	mkfifo(file,0660);
+	SMBGetFile(treeview, (gchar *)get_netfile_cache_dir(), list);
+#else
+	SMBGetFile(treeview, (gchar *)get_netfile_cache_dir(), list);
+	smb_wait(TRUE);
+#endif
+	DBG("got file... (probably)");
+        list = uri_free_list(list);
+
+	return (const gchar *)file;
+}
 
 
+#endif
+
