@@ -1,3 +1,4 @@
+#ifdef USE_SMB_BRANCH
 void
 SMBGetFile (GtkTreeView *treeview, char *target,GList *list);
 void
@@ -8,5 +9,10 @@ SMBrmFile (GtkTreeView *treeview,char *remote_location,
 	       char *remote_pass,char *tmpfile,GList *reference_list);
 void
 SMBmkdir (GtkTreeView *treeview,tree_entry_t *t_en,
-	       GtkTreeIter *iter,char *newdir);
+	       GtkTreeIter *iter,gchar *newdir);
+G_MODULE_EXPORT
+const gchar *SMBget_cache_file(GtkTreeView *treeview, tree_entry_t *en);
+G_MODULE_EXPORT
+const gchar *get_netfile_cache_dir(void);
+#endif
 
