@@ -23,5 +23,10 @@ set_row_colours(	GtkTreeModel * treemodel,
 			GtkTreeRowReference *reference,
 			gint colour_code);
 
+gboolean 
+switch_colors(		GtkTreeModel * treemodel, 
+			GtkTreePath * treepath, 
+			GtkTreeIter * iter, 
+			gpointer data);
 
 #endif
