@@ -382,7 +382,7 @@ void tb_drag_data(GtkWidget * widget, GdkDragContext * context, gint x, gint y,
     if(u->type == URI_SMB)
     {	/* src_host may be remote or local, but target only local
 	  or bookmark */		
- 	g_message("xffm: drop from network not enabled");
+ 	DBG("xffm: drop from network not enabled");
 	print_diagnostics("xfce/error",strerror(EINVAL),"\n",NULL);
 	gtk_drag_finish(context, FALSE, FALSE, time);
 	return;
@@ -910,7 +910,7 @@ void toolbar_buttons(void){
     static gboolean mapped=FALSE;
     GtkWidget *toolbar=WIDGET("toolbar_xftree");
     if (mapped) return;
-    g_message("xffm: loading toolbar buttons");
+    DBG("xffm: loading toolbar buttons");
     smart_buttons(toolbar,exec_button_info,"exec_button");
     smart_buttons(toolbar,exec2_button_info,"exec2_button");
     smart_buttons(toolbar,go_button_info,"go_button");
@@ -1067,7 +1067,7 @@ int parse_panel_config(void){
     g_free(xdg_dir);
     doc = xmlParseFile(rcfile);
     g_free(rcfile);
-    g_message("xffm: loading panel buttons");
+    DBG("xffm: loading panel buttons");
    
     if (!doc) {
 	    return 0;
@@ -1336,7 +1336,7 @@ G_MODULE_EXPORT
 void smart_menus(void){
     static gboolean mapped=FALSE;
     if (mapped) return;
-    g_message("xffm: loading menus");
+    DBG("xffm: loading menus");
     mk_smart_menu(WIDGET("tools1_menu"),exec_button_info);
     mk_smart_menu(WIDGET("open1_menu"),exec2_button_info);
     mk_smart_menu(WIDGET("go1_menu"),go_button_info);
