@@ -33,18 +33,19 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 
-#include "../src/glade_support.h"
+#include <libxfcegui4/xfce_aboutdialog.h>
+
+#include "glade_support.h"
 
 #include "xfdiff_dlg.h"
 #include <stdarg.h>
-#
 #include "xfdiff.h"
 #include "xfdiff_misc.h"
-
+#include "modules.h"
 
 #define COLOR_GDK	65535.0
 
-//#include "icons/xfdiff.xpm"
+/*extern GdkPixbuf *create_pixbuf(gchar *);*/
 
 static GtkTargetEntry target_table[] = {
   {"text/uri-list", 0, TARGET_URI_LIST},
@@ -123,15 +124,12 @@ cb_wheel (GtkWidget * widget, GdkEventButton * event, void *data)
   int new_adj;
 
   new_adj = (GTK_ADJUSTMENT (adj)->value);
-  //printf("button clicked\n"); 
 
   if (event->button == 4){
     new_adj -= (3 * lineH);	
-    //printf("button clicked4\n");
   } 
   if (event->button == 5){
     new_adj += (3 * lineH);	
-    //printf("button clicked5\n");
   }
   if (new_adj < 0)
     new_adj = 0;
@@ -221,7 +219,7 @@ static void
 cb_patchapply (GtkWidget * widget, gpointer data)
 {
   int undoing;
-  undoing = (int) data;
+  undoing = (int) ((long)data);
   if ((!fileD) || (!fileP))
   {
     xf_dlg_warning(diff,("A patch file and directory must be selected first!"));
@@ -658,7 +656,7 @@ cb_do_patch (GtkWidget * widget, gpointer data)
   gchar *s = NULL;
   patched_file *thisF;
   int from_menu;
-  from_menu = (int) data;
+  from_menu = (int) ((long)data);
 
   cleanP ();
   cleanA ();
@@ -789,22 +787,25 @@ void
 cb_save_defaults (GtkWidget * widget, gpointer data)
 {
   FILE *defaults;
-  char *homedir;
-  int len;
+  gchar *xdg_dir=xfce_resource_save_location (XFCE_RESOURCE_CONFIG,"/",TRUE);
+  gchar *rcdir=g_build_filename(xdg_dir,XFDIFF_CONFIG_DIRECTORY,NULL);
+  gchar *rcfile;
 
-  len = strlen ((char *) g_get_home_dir()) + strlen ("/.xfce4/") + strlen (XFDIFF_CONFIG_FILE) + 1;
-  homedir = (char *) malloc ((len) * sizeof (char));
-  if (!homedir)
-    xfdiff_abort (E_MALLOC);
-  snprintf (homedir, len, "%s/.xfce4/%s", (char *) g_get_home_dir(), XFDIFF_CONFIG_FILE);
-  defaults = fopen (homedir, "w");
-  free (homedir);
+  g_free(xdg_dir);
+  mkdir(rcdir, 0770);
+  rcfile=g_build_filename(rcdir,XFDIFF_CONFIG_FILE,NULL);
+  g_free(rcdir);
+  
+  defaults = fopen (rcfile, "w");
 
   if (!defaults)
   {
     xf_dlg_warning(diff,("Default .xfdiffrc file cannot be created\n"));
+    g_free(rcfile);
     return;
   }
+  g_free(rcfile);
+  
   fprintf (defaults, "# file created by xfdiff, if removed xfdiff returns to XFCE defaults.\n");
   fprintf (defaults, "colorfg.red : %d\n", colorfg.red);
   fprintf (defaults, "colorfg.green : %d\n", colorfg.green);
@@ -824,13 +825,35 @@ cb_save_defaults (GtkWidget * widget, gpointer data)
 #ifndef AUTOSAVE
   xf_dlg_warning(diff,("Default configuration has been saved\n"));
 #endif
+  
   return;
 }
 
 static void
-cb_about (GtkWidget * item, GtkWidget * ctree)
+cb_about (GtkWidget * widget, gpointer data)
 {
-  xf_dlg_warning(diff,("This is XFDiff " XFDIFF_VERSION " (xffm-" VERSION ")\n(c) by Edscott Wilson Garcia under GNU-GPL"));
+#ifndef XFCE4_O_X_LIBRARIES
+    XfceAboutInfo *info;
+    GtkWidget *dialog;
+    GdkPixbuf *icon =   MIME_ICON_create_pixbuf ("xfdiff.png");
+    if (!icon) printf("NULL icon\n");
+    info = xfce_about_info_new(
+	    "XFDiff4",
+	    "0.1",
+	    "A patch manager for Xfce4",
+	    XFCE_COPYRIGHT_TEXT("2004", "Edscott Wilson Garcia"),
+	    XFCE_LICENSE_GPL);
+    xfce_about_info_set_homepage(info, "http://www.xfce.org");
+
+    dialog = xfce_about_dialog_new(NULL, info, icon);
+
+    xfce_about_info_free(info);
+
+    gtk_dialog_run(GTK_DIALOG(dialog));
+
+    gtk_widget_destroy(dialog);
+    if (icon) g_object_unref(icon);
+#endif
 }
 
 void
@@ -1108,7 +1131,7 @@ shortcut_menu (int submenu, GtkWidget * parent, char *txt, gpointer func, gpoint
   {
   case TOGGLE:
   case TOGGLENOT:
-    togglevalue = (int) data;
+    togglevalue = (int) ((long)data);
     menuitem = gtk_check_menu_item_new_with_label (txt);
     GTK_CHECK_MENU_ITEM (menuitem)->active = (submenu == TOGGLENOT) ? (!togglevalue) : togglevalue;
     /* gtk_check_menu_item_set_show_toggle (GTK_CHECK_MENU_ITEM (menuitem), 1); */
@@ -1182,14 +1205,14 @@ create_menu (GtkWidget * top)
 /*  shortcut_menu(TOGGLENOT,menu,_("Show diagnostics"),
 		  (gpointer)cb_toggle_diagnostics, (gpointer) silent);*/
   /*shortcut_menu (SUBMENU, menu, _("Font"), (gpointer) cb_select_font, NULL);*/
-  shortcut_menu (SUBMENU, menu, _("Highlight text color"), (gpointer) cb_select_colors, (gpointer) ((long) 0));
-  shortcut_menu (SUBMENU, menu, _("Highlight background color"), (gpointer) cb_select_colors, (gpointer) ((long) 1));
-  shortcut_menu (TOGGLE, menu, _("Fill polygons"), (gpointer) cb_toggle_filledP, (gpointer) filledP);
-  shortcut_menu (TOGGLE, menu, _("Show line numbers"), (gpointer) cb_toggle_show_lineN, (gpointer) show_lineN);
+  shortcut_menu (SUBMENU, menu, _("Highlight text color"), (gpointer) cb_select_colors, (gpointer)  GINT_TO_POINTER(0));
+  shortcut_menu (SUBMENU, menu, _("Highlight background color"), (gpointer) cb_select_colors, (gpointer)  GINT_TO_POINTER(1));
+  shortcut_menu (TOGGLE, menu, _("Fill polygons"), (gpointer) cb_toggle_filledP,  GINT_TO_POINTER(filledP));
+  shortcut_menu (TOGGLE, menu, _("Show line numbers"), (gpointer) cb_toggle_show_lineN, GINT_TO_POINTER(show_lineN));
 
   shortcut_menu (TOGGLE, menu, _("Use line synchronization voids"),
 		 /* new text 1.2.3 */
-		 (gpointer) cb_toggle_synchronize, (gpointer) synchronize);
+		 (gpointer) cb_toggle_synchronize, GINT_TO_POINTER(synchronize));
 
 #ifndef AUTOSAVE
   shortcut_menu (EMPTY_SUBMENU, menu, NULL, NULL, NULL);
@@ -1204,11 +1227,11 @@ create_menu (GtkWidget * top)
 			    NULL, NULL);
 
   /* watchout, REVERSE_TOGGLE is #define patchM[whatever] at xfdiff.h */
-  patchM[3] = shortcut_menu (TOGGLE, menu, _("Assume patch is reversed"), (gpointer) cb_toggle_reversed, (gpointer) reversed);
+  patchM[3] = shortcut_menu (TOGGLE, menu, _("Assume patch is reversed"), (gpointer) cb_toggle_reversed, GINT_TO_POINTER (reversed));
 
   patchM[4] = shortcut_menu (TOGGLE, menu, _("Auto strip level"),
 			     /* new text 1.2.3 */
-			     (gpointer) cb_toggle_autostrip, (gpointer) autostrip);
+			     (gpointer) cb_toggle_autostrip, GINT_TO_POINTER(autostrip));
 
   {
     GSList *group = NULL;
@@ -1321,7 +1344,7 @@ create_diff_window (void)
   diff = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   {
    GdkPixbuf *icon_pixbuf;
-   icon_pixbuf = create_pixbuf ("xfdiff.png");
+   icon_pixbuf = MIME_ICON_create_pixbuf ("xfdiff.png");
    if (icon_pixbuf){
       gtk_window_set_icon (GTK_WINDOW (diff), icon_pixbuf);
       g_object_unref (G_OBJECT(icon_pixbuf));
